//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.fulfillment.commands.*;

import com.ibm.commerce.catalog.commands.*;
import com.ibm.commerce.inventory.objects.*;
import java.text.*;
import java.sql.*;

import com.ibm.commerce.datatype.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.common.objects.*;


import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.user.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.utils.*;

import com.ibm.commerce.price.utils.*;

/**
 * Checks that either there is sufficient existing inventory for a list of items, or that they could be backordered.
 * <p><strong>Behaviour</strong>
 * <p>The input item(s) are updated with estimatedAvailabilityTime and shippingOffset information.<p>
 *
 */
public class CheckInventoryAvailabilityCmdImpl  extends TaskCommandImpl implements CheckInventoryAvailabilityCmd
{
	protected static String istrThisClass                = "com.ibm.commerce.fulfillment.commands.CheckInventoryAvailabilityCmdImpl";	
	protected static String istrPerformFunc              = "performExecute"     ;

	public final static String ERR_CHECK_INV = "1001";
	
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	private Hashtable    ihsOrderItemFulfillmentCentersMaps = new Hashtable();
	private Enumeration  ienOrderItems                      = null;
	private Integer      iStoreId                           = null;

	private com.ibm.commerce.order.calculation.Item          iCatEntry                    = null;
	private Vector                                           ivCatEntryFulfillmentCenters = new Vector(0);
	private String											 istrUnitOfMeasure            = null;

	private InvError     									 iInvError 					  = new InvError();
	
	class DateIndex
	{
		public DateIndex(Timestamp aDate, int aIdx) { iDate = aDate; iIndex = aIdx; };
		private Timestamp iDate  = new Timestamp(0);
		private int       iIndex = -1;

		public Timestamp getDate () { return iDate ; }
		public int       getIndex() { return iIndex; }
		public void      setDate (Timestamp aDate) { iDate = aDate; }
		public void      setIndex(int       aIdx ) { iIndex = aIdx; }
	};

	private class InvError
	{
		public Vector vCatEntries          = new Vector();
		public Vector vQuantities 		   = new Vector();
		public Vector vFulfillmentCenters  = new Vector();
		public Vector vStores   		   = new Vector();
		
		public void add(Long anCatEntryId, Integer anQuantity, Vector avFFCs, Integer anStores)
		{
			vCatEntries        .add(anCatEntryId);
			vQuantities        .add(anQuantity  );
			vFulfillmentCenters.add(avFFCs      );
			vStores            .add(anStores    );
		}

		public boolean inventoryMissing() { return vCatEntries.size() > 0 ? true : false; }
		
		public Vector getCatEntries       () { return vCatEntries        ; }
		public Vector getQuantities       () { return vQuantities        ; }
		public Vector getFufillmentCenters() { return vFulfillmentCenters; }
		public Vector getStores           () { return vStores            ; }		

	};


private Object[] checkInventory(Long anItemSpec, Integer anQuantity, Integer anStoreId, Vector avFFCs) throws ECException
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, "checkInventoryAvailabilty");

	Timestamp timeStamp       = null;
	Integer   nShippingOffset = new Integer(0);
	String strFFCList = InventoryManagementHelper.ffcsToString(avFFCs.elements());
	
	try
	{

		//check availability from the list of fulfillment centers passed in for the orderitem
		TypedProperty tpSPResult = StoredProcedureHelperProxy.callAvailInvStore(anItemSpec, anStoreId, strFFCList);

		String strFFCAvail        = tpSPResult.getString(InventoryManagementHelper.AVAILABLE_INVENTORY_RESP);
		String strQuantitiesAvail = tpSPResult.getString(InventoryManagementHelper.AVAILABLE_INVENTORY_LIST);

		Vector vQuantities = new Vector(avFFCs.size());	 			
		if (totalQuantity(strQuantitiesAvail, vQuantities) >= anQuantity.intValue())
		{			
			//set the timestamp to available now
			timeStamp = TimestampHelper.now();		
			Integer nFFC = getFFCWithNonZeroInventory(vQuantities, avFFCs);
			if (nFFC != null) // should always be the case
			{  
				nShippingOffset = InventoryManagementHelper.getShippingOffset(anItemSpec, nFFC, anStoreId);									
			}

		}
		else
		{			
			tpSPResult              = StoredProcedureHelperProxy.callAvailRADate(anItemSpec, anQuantity, anStoreId, strFFCList);
			String    strRADateList = tpSPResult.getString(InventoryManagementHelper.RA_DATE_LIST);

				
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, "checkInventoryAvailabilty", "RA date list : " + strRADateList);
					
			DateIndex soonestDate = getBestDate(strRADateList);
			int       index       = soonestDate.getIndex();
			if (index > -1)
			{																
					//set the timestamp to the soonest date 
					timeStamp       = soonestDate.getDate();
					nShippingOffset = InventoryManagementHelper.getShippingOffset(anItemSpec, (Integer)avFFCs.elementAt(index), anStoreId);
			}
			else
			{
				// if this order item is not discontinued set estimated availability time to the maximum
				ItemSpecificationAccessBean abItemSpec = new ItemSpecificationAccessBean();
				abItemSpec.setInitKey_itemspcId(anItemSpec.toString());
				if (!abItemSpec.getDiscontinued().equalsIgnoreCase(new String("Y"))) // replace with literal
				{
					StoreAccessBean abStore         = getCommandContext().getStore(anStoreId);
					Integer         nDefaultBOOffset = abStore            .getDefaultBoOffsetInEJBType();
					
					timeStamp       = new Timestamp(TimestampHelper.now().getTime() + nDefaultBOOffset.longValue()*1000);
					nShippingOffset = InventoryManagementHelper.getShippingOffset(anItemSpec, (Integer)avFFCs.elementAt(0), anStoreId);					
				}
								

			}

		}			

	}
	catch (RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, istrThisClass, "checkInventoryAvailabilty", ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, istrThisClass, "checkInventoryAvailabilty", ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, istrThisClass, "checkInventoryAvailabilty", ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, istrThisClass, "checkInventoryAvailabilty", ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (SQLException    e) { throw new ECSystemException(ECMessage._ERR_SQL_EXCEPTION   , istrThisClass, "checkInventoryAvailabilty", ECMessageHelper.generateMsgParms( e.getMessage()), e );}

		

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, "checkInventoryAvailabilty");

	
	return timeStamp != null ? new Object[] {timeStamp, nShippingOffset} : null;
}
	private void checkInventoryForCatEntry() throws ECException
	{
		final String methodName = "checkInventoryForCatEntry";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
		
		Long nCatEntryId =  iCatEntry.getCatalogEntryId(); // assume that this is an item rather than a product; no attributes are needed to resolve an item spec
				
		Long nItemSpecId  = null;

		CatalogEntryAccessBean abCatEntry = new CatalogEntryAccessBean();
		abCatEntry.setInitKey_catalogEntryReferenceNumber(nCatEntryId.toString());

		try
		{
			nItemSpecId = abCatEntry.getItemspc_idInEJBType();
		}
		catch (Exception ex)
		{   
			// failed to resolve item spec; throw an exception that inventory can not be checked
			throw new ECSystemException(ECMessage._ERR_PROD_NOT_EXISTING,this.getClass().getName(),"performExecute",ECMessageHelper.generateMsgParms(nCatEntryId.toString()),ex);			
	    }

		if (nItemSpecId == null)
		{			
			throw new ECApplicationException(ECMessage._ERR_GENERIC, istrThisClass, methodName,  ECMessageHelper.generateMsgParms("No item spec set for catalog entry " + nCatEntryId));
		}		

		// convert given quantity for this order item to normalized quantity and then to inventory quantity
		Double  fQuantity       = Helper.convertToNormalizedQuantity(nCatEntryId, new Double(iCatEntry.getQuantity().doubleValue()), getUnitOfMeasure());
		// convert catalog quantity to inventory quantity
		Integer nNeededQuantity = Helper.calculateInventoryQuantity(fQuantity, nCatEntryId, nItemSpecId);			
		Integer nStoreId        = iCatEntry.getStoreEntityId();
		if (nStoreId == null) 
			nStoreId = getStoreId();

		Object [] arrTimestampShippingOffsetPair = checkInventory(nItemSpecId, nNeededQuantity, nStoreId, getCatEntryFulfillmentCenters());
		if (null == arrTimestampShippingOffsetPair)
		    // inventory check is unsuccessful; item may be discontinued
			iInvError.add(nCatEntryId, nNeededQuantity, getCatEntryFulfillmentCenters(), nStoreId);
		else
		{
			iCatEntry.setEstimatedAvailableTime((Timestamp) arrTimestampShippingOffsetPair[0]);
			iCatEntry.setShippingOffset        ((Integer  ) arrTimestampShippingOffsetPair[1]);
		}

		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
	}
// returns the most recent date from the input list and its position in the list
private DateIndex getBestDate(String astrDateList)
{
	
	DateIndex bestDateAndItsIndex  = new DateIndex(new Timestamp(0), -1);	
	
	Vector         vDates       = parseStringList(astrDateList, InventoryManagementHelper.FFC_LIST_SEPARATOR);
	Enumeration    enDates      = vDates.elements();
	int            idx          = -1;
	while (enDates.hasMoreElements())
	{
		idx++;
		String strDate = (String)enDates.nextElement();
		try
		{
			// The format of the date given to us by the SP is as follows YYYY-MM-DD-HH.MM.SS.
			// We want to convert it to the format accepted by the Timestamp class, i.e., YYYY-MM-DD HH:MM:SS
			StringBuffer sbFormattedDate = new StringBuffer(strDate);
			sbFormattedDate.setCharAt(10, ' ');
			sbFormattedDate.setCharAt(13, ':');
			sbFormattedDate.setCharAt(16, ':');
			sbFormattedDate.setLength(sbFormattedDate.length()-1); // get rid of the trailing period
			Timestamp tsNext =  Timestamp.valueOf(sbFormattedDate.toString());
			if (bestDateAndItsIndex.getDate().before(tsNext))
			{
			  bestDateAndItsIndex.setDate (tsNext);
			  bestDateAndItsIndex.setIndex(idx   );
			}
		}		
		catch (StringIndexOutOfBoundsException ex) { continue; } // ignore strings that can not be parsed as dates
		catch (IllegalArgumentException        ex) { continue; }

	}

	return bestDateAndItsIndex;
		
}
/**
 * Gets an item to check inventory for.
 * @return java.lang.Long Item representing the catalog entry and quantity to check inventory.
 */
public com.ibm.commerce.order.calculation.Item getCatEntry() {
	return iCatEntry;
}
/**
 * Gets a list of fulfillment centers for an item specified by setCatEntry method.
 * 
 */
public java.util.Vector getCatEntryFulfillmentCenters() {
	return ivCatEntryFulfillmentCenters;
}
// input are matching lists of quantities and fulfillment centers
// returns the first fulfillment center with non-zero quantity
private Integer getFFCWithNonZeroInventory(Vector avQuantities, Vector avFFCs)
{
	Integer nFFC = null;
	int     nSz  = avQuantities.size();
	
	for (int idx = 0; idx < nSz && nFFC == null; idx++)
	{
		Integer nQuantity = (Integer)avQuantities.elementAt(idx);
		
		if (nQuantity.longValue() > 0)
		{
			if (avFFCs.size() > idx)
			{
				nFFC = (Integer)avFFCs.elementAt(idx);
			}
		}
	}
			
    return nFFC;
}
/**
 * Gets lists of fulfillment centers to check inventory for items specified by the setOrderItems method. 
 */
public java.util.Hashtable getOrderItemFulfillmentCentersMaps() {
	return ihsOrderItemFulfillmentCentersMaps;
}
/**
 *  Gets items to check inventory. 
 */
public java.util.Enumeration getOrderItems() {
	return ienOrderItems;
}
/**
 * Gets the unit of measure to check inventory for an item specified by setCatEntry method.
 */
public java.lang.String getUnitOfMeasure() {
	return istrUnitOfMeasure;
}
	private void handleErrors() throws ECApplicationException
	{
		if (iInvError.inventoryMissing()) // do not have sufficient inventory for some or all of the products
		{
			TypedProperty errorNVPs = new TypedProperty();

				errorNVPs.put(com.ibm.commerce.server.ECConstants.EC_ERROR_CODE, ERR_CHECK_INV); 
				errorNVPs.put(com.ibm.commerce.server.ECConstants.EC_STORE_ID, iInvError.vStores);
				errorNVPs.put(OrderConstants.EC_CATENTRY_ID          , iInvError.vCatEntries); 
				errorNVPs.put(OrderConstants.EC_QUANTITY             , iInvError.vQuantities);
				errorNVPs.put(OrderConstants.EC_FULFILLMENT_CENTER_ID, iInvError.vFulfillmentCenters);
				
  			    throw new ECApplicationException(ECMessage._API_BAD_INV, this.getClass().getName(), "processErrors",
					ECMessageHelper.generateMsgParms(iInvError.vCatEntries.elementAt(0), iInvError.vQuantities.elementAt(0)), CheckInventoryCmd.ERRTASK_NAME, errorNVPs);
  	    }

	}
	private Vector parseStringList(String astrToParse, String astrSeparator)
	{
		Vector vTokens = new Vector(10);

		int start = 0;
		int end   = 0;
		boolean done = false;
		
		while (!done)
		{
			int separatorLocation = astrToParse.indexOf(astrSeparator, start);			
			if (separatorLocation != -1)			
				end = separatorLocation;				 			
			else
			{
		 		end  = astrToParse.length();
		 		done = true;
			}

			 String strToken = astrToParse.substring(start, end);			
		 	 vTokens.add(strToken.trim());			
			
			start = end + 1;	
		}

		return vTokens;		

	}
/**
 * Contains the actual business logic of the command.
 */
public void performExecute() throws ECException
{
	final String methodName = istrPerformFunc;
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);

	try
	{		
		if (ienOrderItems != null)
		{			
			
			while (ienOrderItems.hasMoreElements())
			{
				OrderItemAccessBean abOrderItem = (OrderItemAccessBean) ienOrderItems.nextElement();
				Long                nItemSpec   = abOrderItem.getItemSpecIdInEJBType    ();
				Integer             nQuantity   = abOrderItem.getNeededQuantityInEJBType();
				Integer             nStoreId    = abOrderItem.getStoreIdInEJBType       ();

				Vector vFFCs = InventoryManagementHelper.getFulfillmentCenters(ihsOrderItemFulfillmentCentersMaps, abOrderItem.getOrderItemIdInEJBType());
				if (vFFCs.size() == 0)
				{
					// no fulfillment centers for this order item; report an error
					throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, 
						istrThisClass, methodName, ECMessageHelper.generateMsgParms("fulfillmentCenterId"));

				}				

				//We need to check if this is a simple order item, or if it has components				
				Enumeration enComponents = abOrderItem.getConfigurationComponents();
				Object [] arrTimestampShippingOffsetPair = new Object []  { (Timestamp) null, new Integer(0) };
				if (null == enComponents || !enComponents.hasMoreElements())
				{
					//there are no components, so we perfrom a normal check inv
					arrTimestampShippingOffsetPair = checkInventory(nItemSpec, nQuantity, nStoreId, vFFCs);
					if (null == arrTimestampShippingOffsetPair)
					{   // inventory check is unsuccessful; item may be discontinued
						iInvError.add(abOrderItem.getCatalogEntryIdInEJBType(), nQuantity, vFFCs, nStoreId);
						continue;
					}
					abOrderItem.setEstimatedAvailableTime((Timestamp)arrTimestampShippingOffsetPair[0]);
					abOrderItem.setShippingOffset        ((Integer  )arrTimestampShippingOffsetPair[1]);
					abOrderItem.commitCopyHelper         ();
					
				}
				else
				{
					//There are components, so we need to loop through them and check the inventory for each
					Timestamp t = null;
					Timestamp temp;
					while (enComponents.hasMoreElements())
					{
						OrderItemComponentAccessBean abOIC = (OrderItemComponentAccessBean) enComponents.nextElement();
						nItemSpec = abOIC.getItemSpecificationIdInEJBType();
						nQuantity = new Integer(nQuantity.intValue() * abOIC.getInventoryQuantityInEJBType().intValue());
						
						arrTimestampShippingOffsetPair = checkInventory(nItemSpec, nQuantity, nStoreId, vFFCs);
						if (null == arrTimestampShippingOffsetPair)
						{   // inventory check is unsuccessful; item may be discontinued 
							iInvError.add(abOIC.getCatalogEntryIdInEJBType(), nQuantity, vFFCs, nStoreId);	// is store id available from the order item component
							continue;
						}
						temp = (Timestamp) arrTimestampShippingOffsetPair[0];

						ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName,
							"The earliest fulfillment time for item spec: " + nItemSpec.toString() + " is " + temp);
							

						//The estimated availability time for the order item is the longest availability time of
						//any of the components
						if ((temp != null) && (t == null || t.before(temp)))
						{
							ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName,
							"Setting kit inventory check time to: " + temp);
							t = temp;
						}
					}

					if (null == arrTimestampShippingOffsetPair)
						continue;
						
					abOrderItem.setEstimatedAvailableTime(t);
					abOrderItem.setShippingOffset        ((Integer)arrTimestampShippingOffsetPair[1]);
					abOrderItem.commitCopyHelper         ();				
				}

			} // while

		}	// if 

		if (getCatEntry() != null)
		{ // check inventory for given catalog entry
			checkInventoryForCatEntry();
		}

		handleErrors();
			


	}
	catch (RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
		

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
}
/**
 * This method should be called after a command has been executed to reset its states variables.
 */
public void reset() 
	{ 
		ihsOrderItemFulfillmentCentersMaps = new Hashtable();
		ienOrderItems                      = null;
			
		iStoreId                           = null;
		iCatEntry                          = null;
		ivCatEntryFulfillmentCenters       = new Vector  (0);
		iInvError                          = new InvError();

	}
/**
 * Sets an item to check the inventory.
 * @param 	aCatEntry	Item representing the catalog entry and quantity to check inventory.
 */
public void setCatEntry(com.ibm.commerce.order.calculation.Item newCatEntry) {
	iCatEntry = newCatEntry;
}
/**
 * Sets a list of fulfillment centers for an item specified by setCatEntry method.
 * @param 	avCatEntryFulfillmentCenters	list of fulfillment centers.
 */	
public void setCatEntryFulfillmentCenters(java.util.Vector avCatEntryFulfillmentCenters) {
	ivCatEntryFulfillmentCenters = avCatEntryFulfillmentCenters;
}
/**
* Sets lists of fulfillment centers to check inventory for items specified by the setOrderItems method. 
*	@param 		ahsOrderItemFulfillmentCentersMaps	lists of fulfillment centers keyed by the order item id
*/

public void setOrderItemFulfillmentCentersMaps(java.util.Hashtable newOrderItemFulfillmentCentersMaps) {
	ihsOrderItemFulfillmentCentersMaps = newOrderItemFulfillmentCentersMaps;
}
/**
*  Sets items to check inventory.  Each item has an associated prioritized list of fulfillment centers where the check should be made
*  specified by the setOrderItemFulfillmentCentersMaps method.
*  @param	aenOrderItems	items to check inventory
*/
public void setOrderItems(java.util.Enumeration newOrderItems) {
	ienOrderItems = newOrderItems;
}
/**
 * Set a store id to check the inventory.
 *
 */
public void setStoreId(java.lang.Integer newStoreId) {
	iStoreId = newStoreId;
}
/**
* Sets the unit of measure to check inventory for an item specified by setCatEntry method.
* @param	astrUnitOfMeasure	unit of measure.
*/
public void setUnitOfMeasure(java.lang.String astrUnitOfMeasure) {
	istrUnitOfMeasure = astrUnitOfMeasure;
}
// parse comma separate integer values and add them up
// if invalid value occurs ignore it	
private int totalQuantity(String astrQuantities, Vector avQuantities)
{
	int    start = 0;
	int    end   = 0;
	int    total = 0;
		
	boolean done = false;
		
		while (!done)
		{
			int separatorLocation = astrQuantities.indexOf(InventoryManagementHelper.FFC_LIST_SEPARATOR, start);
			String strInteger;
			if (separatorLocation != -1)			
				end = separatorLocation;				 			
			else
			{
		 		end  = astrQuantities.length();
		 		done = true;
			}

			 strInteger = astrQuantities.substring(start, end).trim();

			try
			{
				Integer nValue = new Integer(strInteger);
				avQuantities.add(nValue);
				total += nValue.intValue();
			}
			catch (NumberFormatException ex)
			{ // ignore invalidly formatted numbers
			}
			
			start = end + 1;	

		}

		return total;	

}
}
