//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.fulfillment.commands.*;

import java.sql.*;

import com.ibm.commerce.datatype.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.common.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.user.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.utils.*;

/**
 * Deallocates expected inventory for a particular order item at a particular fulfillment center.
 * <p><strong>Behaviour</strong>
 * <p> If the order item is currently backordered, the amount backordered is returned back to expected inventory.  
 *     The order item is updated to indicate that it is no longer backordered.
 * <p> Otherwise, the order item is not changed.
 */
public class DeallocateExpectedInventoryCmdImpl extends TaskCommandImpl implements DeallocateExpectedInventoryCmd
{
	public static final String COPYRIGHT                 = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	protected static String istrThisClass                = "com.ibm.commerce.order.commands.DeallocateExpectedInventoryCmdImpl";	
	protected static String istrPerformFunc              = "performExecute";
	protected static String istrCheckParametersFunc      = "checkParameters";

	private OrderItemAccessBean    iabOrderItem            = null;	
	private ReverseInventoryHelper iReverseInventoryHelper = new ReverseInventoryHelper();

/**
 * Gets an order item for which expected inventory is to be deallocated.
 *
 */
public com.ibm.commerce.order.objects.OrderItemAccessBean getOrderItem() {
	return iabOrderItem;
}
/**
 * Contains the actual business logic of the command.
 */
public void performExecute() throws ECException
{
	final String methodName = istrPerformFunc;
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);

//	checkParameters();

	// CH: Rob suggested to be add capability to be able to specify quantity to deallocate.
	// Stored procedures do not provide this capability? Do we really need it? Investigate...
	try
	{
		// we are delegating the reverse of inventory allocation to a helper method since
		// the code is the same for expected and existing inventory deallocation
		iReverseInventoryHelper.reverseInventory(getOrderItem());

	}
	catch (RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (SQLException    e) { throw new ECSystemException(ECMessage._ERR_SQL_EXCEPTION   , istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
}
/**
 * Sets an order item for which expected inventory is to be deallocated.
 *
 */
public void setOrderItem(com.ibm.commerce.order.objects.OrderItemAccessBean newOrderItem) {
	iabOrderItem = newOrderItem;
}
/**
 * Performs server side parameter checking.
 */
public void validateParameters() throws com.ibm.commerce.exception.ECException {
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, istrCheckParametersFunc);

	if (getOrderItem() == null)
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, istrThisClass, istrCheckParametersFunc, ECMessageHelper.generateMsgParms(OrderConstants.EC_ORDER_ID));

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, istrCheckParametersFunc);
}
}
