//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.membergroup.commands.*;

import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.command.TaskCommandImpl;
import com.ibm.commerce.common.objects.LanguageAccessBean;
import com.ibm.commerce.common.objects.StoreAccessBean;
import com.ibm.commerce.condition.Condition;
import com.ibm.commerce.condition.ConditionConstants;
import com.ibm.commerce.condition.ConditionUtil;
import com.ibm.commerce.condition.OpenCondition;
import com.ibm.commerce.condition.SimpleCondition;
import com.ibm.commerce.contract.helper.ECContractConstants;
import com.ibm.commerce.contract.objects.ParticipantAccessBean;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.member.helpers.OrgEntityManageBean;
import com.ibm.commerce.member.helpers.UserManageBean;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.payment.objects.AccountAccessBean;
import com.ibm.commerce.price.utils.CurrencyManager;
import com.ibm.commerce.price.utils.MonetaryAmount;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.server.WcsApp;
import com.ibm.commerce.tools.segmentation.SegmentCache;
import com.ibm.commerce.tools.segmentation.SegmentConstants;
import com.ibm.commerce.tools.segmentation.SegmentUtil;
import com.ibm.commerce.tools.segmentation.SegmentXMLConditionConverter;
import com.ibm.commerce.user.objects.AddressAccessBean;
import com.ibm.commerce.user.objects.DemographicsAccessBean;
import com.ibm.commerce.user.objects.MemberGroupAccessBean;
import com.ibm.commerce.user.objects.MemberGroupMemberAccessBean;
import com.ibm.commerce.user.objects.MemberRoleAccessBean;
import com.ibm.commerce.user.objects.OrganizationAccessBean;
import com.ibm.commerce.user.objects.RoleAccessBean;
import com.ibm.commerce.user.objects.UserAccessBean;
import com.ibm.commerce.user.objects.UserProfileAccessBean;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.Enumeration;
import java.util.Vector;

/**
 * This class implements the CheckUserInMemberGroupCmd interface to
 * provide the task command that checks if a specified user belongs to a specified member group.
 */
public class CheckUserInMemberGroupCmdImpl extends TaskCommandImpl implements CheckUserInMemberGroupCmd,
                                                                              SegmentConstants,
                                                                              ConditionConstants
 {
  /**
   * Copyright.
   */
  public static final String COPYRIGHT = IBMCopyright.SHORT_COPYRIGHT;

  /**
   * An instance of UserAccessBean.
   */
  protected UserAccessBean _user = null;

  /**
   * An instance of UserManageBean.
   */
  protected UserManageBean _userManageBean = null;

  /**
   * The member group name.
   */
  protected String _memberGroupName = null;

  /**
   * The member group owner id.
   */
  protected Long _memberGroupOwnerId = null;

  /**
   * An instance of MemberGroupAccessBean.
   */
  protected MemberGroupAccessBean _memberGroupAccessBean = null;

  /**
   * The result of this check.
   */
  protected boolean _result = false;

  /**
   * An instance of DemographicsAccessBean.
   */
  protected DemographicsAccessBean _demographics = null;

  /**
   * An instance of AddressAccessBean.
   */
  protected AddressAccessBean _address = null;

  /**
   * An instance of UserProfileAccessBean.
   */
  protected UserProfileAccessBean _userProfile = null;

  /**
   * A vector of member groups IDs to which this user belongs.
   */
  protected Vector _memberGroups = null;

  /**
   * A vector of role names for this user.
   */
  protected Vector _roles = null;

  /**
   * A vector of role names within the an organization.
   */
  protected Vector _orgRoles = null;

  /**
   * The ID of the organization that matches _orgRoles.
   */
  protected Long _orgRolesOrgId = null;

  /**
   * An instance of CheckUserInMemberGroup.
   */
  protected CheckUserInMemberGroupCmd _checkUserInMemberGroupCmd = null;

  /**
   * The instance of CheckUserInMemberGroupCmdImpl that called this
   * instance.  This is used to prevent infinite recursion.
   */
  protected CheckUserInMemberGroupCmdImpl _parent = null;

  /**
   * The member group condition object.
   */
  protected Condition _memberGroupCondition = null;

  /**
   * An instance of OrgEntityManageBean.
   */
  protected OrgEntityManageBean _orgEntityManageBean = null;

  public void setUser(UserAccessBean user)
   {
    _user = user;
    _userManageBean = null;
    _address = null;
    _userProfile = null;
    _demographics = null;
    _memberGroups = null;
    _roles = null;
    _orgRoles = null;
    _orgRolesOrgId = null;
    _orgEntityManageBean = null;
   }

  public void setMemberGroupName(String memberGroupName)
   {
    _memberGroupName = memberGroupName;
   }

  public void setMemberGroupOwnerId(Long memberGroupOwnerId)
   {
    _memberGroupOwnerId = memberGroupOwnerId;
   }

  public void setMemberGroupAccessBean(MemberGroupAccessBean memberGroupAccessBean)
   {
    _memberGroupAccessBean = memberGroupAccessBean;
   }

  public void setMemberGroupCondition(Condition memberGroupCondition)
   {
    _memberGroupCondition = memberGroupCondition;
   }

  public boolean getResult()
   {
    return _result;
   }

  /**
   * Checks to see if the command is ready to call.  This method will return
   * false if both the member group name and the member group access bean are null or
   * if the user is null.
   *
   * @return <code>true</code> if the command is ready to go.
   */
  public boolean isReadyToCallExecute()
   {
    return super.isReadyToCallExecute() && _user != null && (_memberGroupName != null || _memberGroupAccessBean != null);
   }

  /**
   * Performs the execution of the command.
   */
  public void performExecute() throws ECException
   {
    final String methodName = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_USER, getClass().getName(), methodName);
    if (_memberGroupAccessBean != null)
     {
      try
       {
        _memberGroupName = _memberGroupAccessBean.getMbrGrpName();
        _memberGroupOwnerId = _memberGroupAccessBean.getOwnerIdInEJBType();
       }
      catch (Exception e)
       {
       }
     }
    if (_memberGroupOwnerId == null)
     {
      try
       {
        _memberGroupOwnerId = getCommandContext().getStore().getMemberIdInEJBType();
       }
      catch (Exception e)
       {
       }
     }
    Long memberId = null;
    try
     {
      memberId = _user.getMemberIdInEJBType();
     }
    catch (Exception e)
     {
     }
    Vector explicitlyIncludedMembers = SegmentCache.getSegmentExplicitlyIncludedMembers(_memberGroupOwnerId, _memberGroupName);
    Vector explicitlyExcludedMembers = SegmentCache.getSegmentExplicitlyExcludedMembers(_memberGroupOwnerId, _memberGroupName);
    if (_memberGroupCondition == null)
     {
      _memberGroupCondition = SegmentCache.getSegmentCondition(_memberGroupOwnerId, _memberGroupName);
     }
    boolean explicitMembership = false;
    if (explicitlyExcludedMembers != null)
     {
      for (int i = 0; i < explicitlyExcludedMembers.size(); i++)
       {
        if (explicitlyExcludedMembers.elementAt(i).equals(memberId))
         {
          _result = false;
          explicitMembership = true;
          break;
         }
       }
     }
    if (!explicitMembership)
     {
      if (explicitlyIncludedMembers != null)
       {
        for (int i = 0; i < explicitlyIncludedMembers.size(); i++)
         {
          if (explicitlyIncludedMembers.elementAt(i).equals(memberId))
           {
            _result = true;
            explicitMembership = true;
            break;
           }
         }
       }
     }
    if (!explicitMembership)
     {
      if (_memberGroupCondition != null)
       {
        _result = _memberGroupCondition.evaluate(this);
       }
      else
       {
        _result = false;
       }
     }
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_USER, getClass().getName(), methodName);
   }

  /**
   * Evaluate a simple condition.  Simple conditions are persisted in XML format as follows:
   * <code><br>
   * &LT;simpleCondition><br>
   *   &LT;variable name="gender"/><br>
   *   &LT;operator name="="/><br>
   *   &LT;value data="M"/><br>
   * &LT;/simpleCondition><br>
   * </code>
   *
   * @param variable  The name of the variable.
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifires  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  public boolean evaluate(String variable, String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    boolean result = false;

    if (VARIABLE_REGISTRATION_STATUS.equals(variable))
     {
      result = evaluateRegistrationStatusCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_DAYS_SINCE_REGISTRATION.equals(variable))
     {
      result = evaluateDaysSinceRegistrationCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_REGISTRATION_DATE.equals(variable))
     {
      result = evaluateRegistrationDateCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_DAYS_SINCE_REGISTRATION_CHANGE.equals(variable))
     {
      result = evaluateDaysSinceRegistrationChangeCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_REGISTRATION_CHANGE_DATE.equals(variable))
     {
      result = evaluateRegistrationChangeDateCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_GENDER.equals(variable))
     {
      result = evaluateGenderCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_AGE_GROUP.equals(variable))
     {
      result = evaluateAgeGroupCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_INCOME_GROUP.equals(variable))
     {
      result = evaluateIncomeGroupCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_MARITAL_STATUS.equals(variable))
     {
      result = evaluateMaritalStatusCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_CHILDREN.equals(variable))
     {
      result = evaluateChildrenCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_HOUSEHOLD.equals(variable))
     {
      result = evaluateHouseholdCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_CITY.equals(variable))
     {
      result = evaluateCityCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_STATE.equals(variable))
     {
      result = evaluateStateCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_COUNTRY.equals(variable))
     {
      result = evaluateCountryCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ZIP_CODE.equals(variable))
     {
      result = evaluateZipCodeCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_PHONE.equals(variable))
     {
      result = evaluatePhoneCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_E_MAIL.equals(variable))
     {
      result = evaluateEMailCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_CURRENCY.equals(variable))
     {
      result = evaluateCurrencyCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_LANGUAGE.equals(variable))
     {
      result = evaluateLanguageCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_LOCALE.equals(variable))
     {
      result = evaluateLocaleCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_AMOUNT_SPENT.equals(variable))
     {
      result = evaluateAmountSpentCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ORDERS.equals(variable))
     {
      result = evaluateOrdersCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_DAYS_SINCE_LAST_PURCHASE.equals(variable))
     {
      result = evaluateDaysSinceLastPurchaseCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_LAST_PURCHASE_DATE.equals(variable))
     {
      result = evaluateLastPurchaseDateCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_DAYS_SINCE_LAST_VISIT.equals(variable))
     {
      result = evaluateDaysSinceLastVisitCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_LAST_VISIT_DATE.equals(variable))
     {
      result = evaluateLastVisitDateCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_COMPANY.equals(variable))
     {
      result = evaluateCompanyCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_INTERESTS.equals(variable))
     {
      result = evaluateInterestsCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_MEMBER_GROUPS.equals(variable))
     {
      result = evaluateMemberGroupsCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_SEGMENT.equals(variable))
     {
      result = evaluateSegmentCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_PREFERRED_COMMUNICATION.equals(variable))
     {
      result = evaluatePreferredCommunicationCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ROLE.equals(variable))
     {
      result = evaluateRoleCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_STATUS.equals(variable))
     {
      result = evaluateStatusCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ORG.equals(variable))
     {
      result = evaluateOrgCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ORG_DN.equals(variable))
     {
      result = evaluateOrgDNCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_JOB_FUNCTION.equals(variable))
     {
      result = evaluateJobFunctionCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT.equals(variable))
     {
      result = evaluateAccountCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_SIZE.equals(variable))
     {
      result = evaluateAccountSizeCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_LOCATION.equals(variable))
     {
      result = evaluateAccountLocationCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_COUNTRY.equals(variable))
     {
      result = evaluateAccountCountryCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_BUSINESS.equals(variable))
     {
      result = evaluateAccountBusinessCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_INDUSTRY.equals(variable))
     {
      result = evaluateAccountIndustryCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_CREDIT.equals(variable))
     {
      result = evaluateAccountCreditCondition(operator, value, qualifiers);
     }
    else if (VARIABLE_ACCOUNT_AMOUNT_SPENT.equals(variable))
     {
      result = evaluateAccountAmountSpentCondition(operator, value, qualifiers);
     }
    return result;
   }

  /**
   * Evaluate an open condition.  Open conditions are persisted in XML format as follows:
   * <code><br>
   * &LT;openCondition name="testCondition"><br>
   *   &LT;parameter name="parameter1" value="100"/><br>
   * &LT;/openCondition><br>
   * </code>
   *
   * @param name  The name of the open condition.
   * @param parameters  An array of parameters.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  public boolean evaluate(String name, OpenCondition.Parameter[] parameters)
   {
    boolean result = false;
    if (OPEN_CONDITION_ROLE_LIST.equals(name))
     {
      result = evaluateRoleListCondition(parameters);
     }
    return result;
   }

  /**
   * Evaluate the "registrationStatus" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateRegistrationStatusCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String registerType = "";
    try
     {
      registerType = _user.getRegisterType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(registerType, operator, value);
   }

  /**
   * Evaluate the "daysSinceRegistration" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateDaysSinceRegistrationCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Timestamp registration = null;
    try
     {
      registration = _user.getRegistrationInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDurationInDays(registration, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "registrationDate" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateRegistrationDateCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String registrationDate = null;
    try
     {
      registrationDate = SegmentUtil.toStringDate(_user.getRegistrationInEJBType());
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDate(registrationDate, operator, value);
   }

  /**
   * Evaluate the "daysSinceRegistrationChange" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateDaysSinceRegistrationChangeCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Timestamp registrationUpdate = null;
    try
     {
      registrationUpdate = _user.getRegistrationUpdateInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDurationInDays(registrationUpdate, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "registrationChangeDate" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateRegistrationChangeDateCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String registrationChangeDate = null;
    try
     {
      registrationChangeDate = SegmentUtil.toStringDate(_user.getRegistrationUpdateInEJBType());
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDate(registrationChangeDate, operator, value);
   }

  /**
   * Evaluate the "gender" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateGenderCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String gender = null;
    try
     {
      gender = getDemographics().getGender();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(gender, operator, value);
   }

  /**
   * Evaluate the "ageGroup" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAgeGroupCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Integer age = null;
    try
     {
      age = getDemographics().getAgeInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateInteger(age, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "incomeGroup" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateIncomeGroupCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Integer income = null;
    try
     {
      income = getDemographics().getIncomeInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateInteger(income, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "maritalStatus" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateMaritalStatusCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String maritalStatus = null;
    try
     {
      maritalStatus = getDemographics().getMaritalStatus();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(maritalStatus, operator, value);
   }

  /**
   * Evaluate the "children" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateChildrenCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Integer children = null;
    try
     {
      children = getDemographics().getNumberOfChildrenInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateInteger(children, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "household" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateHouseholdCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Integer household = null;
    try
     {
      household = getDemographics().getNumberOfHouseholdsInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateInteger(household, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "children" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateCityCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String city = null;
    try
     {
      city = getAddress().getCity();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(city, operator, value);
   }

  /**
   * Evaluate the "state" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateStateCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String state = null;
    try
     {
      state = getAddress().getState();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(state, operator, value);
   }

  /**
   * Evaluate the "country" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateCountryCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String country = null;
    try
     {
      country = getAddress().getCountry();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(country, operator, value);
   }

  /**
   * Evaluate the "zipCode" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateZipCodeCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String zipCode = null;
    try
     {
      zipCode = getAddress().getZipCode();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(zipCode, operator, value);
   }

  /**
   * Evaluate the "phone" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluatePhoneCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String phone1 = null;
    String phone2 = null;
    try
     {
      phone1 = getAddress().getPhone1();
      phone2 = getAddress().getPhone1();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluate2Strings(SegmentUtil.stripNonAlphanumericCharacters(phone1),
                                          SegmentUtil.stripNonAlphanumericCharacters(phone2),
                                          operator,
                                          SegmentUtil.stripNonAlphanumericCharacters(value));
   }

  /**
   * Evaluate the "eMail" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateEMailCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String email1 = null;
    String email2 = null;
    try
     {
      email1 = getAddress().getEmail1();
      email2 = getAddress().getEmail1();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluate2Strings(email1, email2, operator, value);
   }

  /**
   * Evaluate the "currency" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateCurrencyCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String currency = null;
    try
     {
      currency = _user.getPreferredCurrency();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(currency, operator, value);
   }

  /**
   * Evaluate the "language" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateLanguageCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String language = null;
    try
     {
      String languageId = _user.getPreferredLanguageId();
      LanguageAccessBean languageAccessBean = new LanguageAccessBean();
      languageAccessBean.setInitKey_languageId(languageId);
      language = languageAccessBean.getLanguage();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(language, operator, value);
   }

  /**
   * Evaluate the "locale" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateLocaleCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String locale = null;
    try
     {
      String languageId = _user.getPreferredLanguageId();
      LanguageAccessBean languageAccessBean = new LanguageAccessBean();
      languageAccessBean.setInitKey_languageId(languageId);
      locale = languageAccessBean.getLocaleName();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(locale, operator, value);
   }

  /**
   * Evaluate the "amountSpent" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAmountSpentCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    BigDecimal amountSpent = new BigDecimal(0.0d);
    try
     {
      StoreAccessBean storeAccessBean = WcsApp.storeRegistry.find(getStoreId());
      CurrencyManager currencyManager = CurrencyManager.getInstance();
      String defaultCurrency = currencyManager.getDefaultCurrency(storeAccessBean);

      Enumeration list = new OrderAccessBean().findWithPushDownQuery("STATUS NOT IN ('P','X','I','E','W','N','M') AND MEMBER_ID=" + _user.getMemberId() + " AND STOREENT_ID=" + getStoreId());
      while (list.hasMoreElements())
       {
        OrderAccessBean orderAccessBean = (OrderAccessBean) list.nextElement();
        String orderCurrency = orderAccessBean.getCurrency();
        BigDecimal orderAmount = orderAccessBean.getTotalProductPriceInEJBType();
        BigDecimal adjustment = orderAccessBean.getTotalAdjustmentInEJBType();
        if (adjustment != null)
         {
          orderAmount = orderAmount.add(adjustment);
         }
        if (orderCurrency.equals(defaultCurrency))
         {
          amountSpent = amountSpent.add(orderAmount);
         }
        else
         {
          MonetaryAmount convertedAmount = currencyManager.convert(new MonetaryAmount(orderAmount, orderCurrency), storeAccessBean, defaultCurrency);
          amountSpent = amountSpent.add(convertedAmount.getValue());
         }
       }
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateBigDecimal(amountSpent, operator, SegmentUtil.toBigDecimal(value));
   }

  /**
   * Evaluate the "orders" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateOrdersCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Integer numberOfOrders = new Integer(0);
    try
     {
      int count = 0;
      Enumeration list = new OrderAccessBean().findWithPushDownQuery("STATUS NOT IN ('P','X','I','E','W','N','M') AND MEMBER_ID=" + _user.getMemberId() + " AND STOREENT_ID=" + getStoreId());
      while (list.hasMoreElements())
       {
        list.nextElement();
        count++;
       }
      numberOfOrders = new Integer(count);
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateInteger(numberOfOrders, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "daysSinceLastPurchase" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateDaysSinceLastPurchaseCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Timestamp lastOrder = null;
    try
     {
      lastOrder = _user.getLastOrderInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDurationInDays(lastOrder, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "lastPurchaseDate" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateLastPurchaseDateCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String lastPurchaseDate = null;
    try
     {
      lastPurchaseDate = SegmentUtil.toStringDate(_user.getLastOrderInEJBType());
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDate(lastPurchaseDate, operator, value);
   }

  /**
   * Evaluate the "daysSinceLastVisit" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateDaysSinceLastVisitCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Timestamp lastSession = null;
    try
     {
      lastSession = _user.getLastSessionInEJBType();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDurationInDays(lastSession, operator, SegmentUtil.toInteger(value));
   }

  /**
   * Evaluate the "lastVisitDate" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateLastVisitDateCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String lastVisitDate = null;
    try
     {
      lastVisitDate = SegmentUtil.toStringDate(_user.getLastSessionInEJBType());
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateDate(lastVisitDate, operator, value);
   }

  /**
   * Evaluate the "company" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateCompanyCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String companyName = null;
    try
     {
      companyName = getDemographics().getCompanyName();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(companyName, operator, value);
   }

  /**
   * Evaluate the "interests" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateInterestsCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String hobbies = null;
    try
     {
      hobbies = getDemographics().getHobbies();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateStringIgnoreCase(hobbies, operator, value);
   }

  /**
   * Evaluate the "memberGroups" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateMemberGroupsCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    Vector memberGroups = getMemberGroups();
    boolean result = false;
    for (int i = 0; i < memberGroups.size(); i++)
     {
      if (memberGroups.elementAt(i).equals(value))
       {
        result = true;
       }
     }
    if (operator.equals(OPERATOR_NOT_EQUAL_TO))
     {
      result = !result;
     }
    return result;
   }

  /**
   * Evaluate the "segment" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateSegmentCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    boolean result = false;
    boolean recursion = false;
    for (CheckUserInMemberGroupCmdImpl parent = _parent; parent != null; parent = parent._parent)
     {
      if (parent._memberGroupName.equals(value))
       {
        recursion = true;
        break;
       }
     }
    if (!recursion)
     {
      CheckUserInMemberGroupCmd checkUserInMemberGroupCmd = getCheckUserInMemberGroupCmd();
      checkUserInMemberGroupCmd.setMemberGroupName(value);
      checkUserInMemberGroupCmd.setMemberGroupCondition(null);
      checkUserInMemberGroupCmd.setUser(_user);
      try
       {
        checkUserInMemberGroupCmd.execute();
        result = checkUserInMemberGroupCmd.getResult();
       }
      catch (Exception e)
       {
       }
     }
    if (OPERATOR_NOT_EQUAL_TO.equals(operator))
     {
      result = !result;
     }
    return result;
   }

  /**
   * Evaluate the "preferredCommunication" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluatePreferredCommunicationCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String preferredCommunication = null;
    try
     {
      preferredCommunication = _user.getUserProfile().getPreferredCommunication();
      if (preferredCommunication != null && preferredCommunication.length() > 1)
       {
        preferredCommunication = preferredCommunication.substring(0, 1);
       }
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(preferredCommunication, operator, value);
   }

  /**
   * Evaluate the "role" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateRoleCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    boolean result = false;
    String org = null;
    String orgDN = null;
    if (qualifiers != null)
     {
      for (int i = 0; i < qualifiers.length; i++)
       {
        String qualifierName = qualifiers[i].getName();
        if (QUALIFIER_ORG.equals(qualifierName))
         {
          org = qualifiers[i].getData();
         }
        else if (QUALIFIER_ORG_DN.equals(qualifierName))
         {
          orgDN = qualifiers[i].getData();
         }
       }
     }
    Vector roles = null;
    if (org != null)
     {
      Long orgId = null;
      if (VALUE_STORE.equals(org))
       {
        try
         {
          orgId = getCommandContext().getStore().getOwner();
         }
        catch (Exception e)
         {
         }
       }
      else
       {
        try
         {
          orgId = new Long(org);
         }
        catch (Exception e)
         {
         }
       }
      roles = getRoles(orgId);
     }
    else if (orgDN != null)
     {
      try
       {
        OrganizationAccessBean organizationAccessBean = new OrganizationAccessBean().findByDN(orgDN);
        roles = getRoles(organizationAccessBean.getOrganizationIdInEJBType());
       }
      catch (Exception e)
       {
       }
     }
    else
     {
      roles = getRoles();
     }
    if (roles != null)
     {
      for (int i = 0; i < roles.size(); i++)
       {
        if (roles.elementAt(i).equals(value))
         {
          result = true;
         }
       }
     }
    if (operator.equals(OPERATOR_NOT_EQUAL_TO))
     {
      result = !result;
     }
    return result;
   }

  /**
   * Evaluate the "status" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateStatusCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String state = "";
    try
     {
      state = _user.getState();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(state, operator, value);
   }

  /**
   * Evaluate the "org" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateOrgCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String parentMemberId = "";
    try
     {
      parentMemberId = _user.getParentMemberId();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(parentMemberId, operator, value);
   }

  /**
   * Evaluate the "orgDN" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateOrgDNCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String parentDN = "";
    try
     {
      String parentMemberId = _user.getParentMemberId();
      OrganizationAccessBean organizationAccessBean = new OrganizationAccessBean();
      organizationAccessBean.setInitKey_MemberId(parentMemberId);
      parentDN = organizationAccessBean.getDistinguishedName();
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateString(parentDN, operator, value);
   }

  /**
   * Evaluate the "jobFunction" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateJobFunctionCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String jobFunction = "";
    UserManageBean userManageBean = getUserManageBean();
    Vector attributeVector = userManageBean.getAttribute(MEMBER_ATTRIBUTE_JOB_FUNCTION, getStoreId().toString());
    if (attributeVector != null && attributeVector.size() > 0)
     {
      jobFunction = (String) attributeVector.lastElement();
     }
    else
     {
    	attributeVector = userManageBean.getAttribute(MEMBER_ATTRIBUTE_JOB_FUNCTION, null);
    	if (attributeVector != null && attributeVector.size() > 0)
     	{
      		jobFunction = (String) attributeVector.lastElement();
     	}
   	 }

    return ConditionUtil.evaluateString(jobFunction, operator, value);
   }

  /**
   * Evaluate the "account" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    boolean result = false;
    Long accountOrgId = getAccountOrganizationId(value);
    if (accountOrgId != null)
     {
      Long parentMemberId = null;
      try
       {
        parentMemberId = new Long(_user.getParentMemberId());
       }
      catch (Exception e)
       {
       }
      if (parentMemberId != null)
       {
        if (accountOrgId.equals(parentMemberId))
         {
          result = true;
         }
        else
         {
          OrgEntityManageBean orgEntityManageBean = new OrgEntityManageBean();
          orgEntityManageBean.setInitKey(parentMemberId.toString());
          try
           {
            Long[] ancestors = orgEntityManageBean.getAncestors();
            if (ancestors != null)
             {
              for (int i = 0; i < ancestors.length; i++)
               {
                if (accountOrgId.equals(ancestors[i]))
                 {
                  result = true;
                 }
               }
             }
           }
          catch (Exception e)
           {
           }
         }
       }
     }
    if (operator.equals(OPERATOR_NOT_EQUAL_TO))
     {
      result = !result;
     }
    return result;
   }

  /**
   * Evaluate the "accountSize" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountSizeCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String accountSize = "";
    OrgEntityManageBean orgEntityManageBean = getOrgEntityManageBean();
    Vector attributeVector = orgEntityManageBean.getAttribute(MEMBER_ATTRIBUTE_ORG_ENTITY_SIZE, null);
    if (attributeVector != null && attributeVector.size() > 0)
     {
      accountSize = (String) attributeVector.elementAt(0);
     }
    return ConditionUtil.evaluateString(accountSize, operator, value);
   }

  /**
   * Evaluate the "accountLocation" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountLocationCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String accountLocation = "";
    OrgEntityManageBean orgEntityManageBean = getOrgEntityManageBean();
    Vector attributeVector = orgEntityManageBean.getAttribute(MEMBER_ATTRIBUTE_ORG_ENTITY_GEOGRAPHIC_LOCATION, null);
    if (attributeVector != null && attributeVector.size() > 0)
     {
      accountLocation = (String) attributeVector.elementAt(0);
     }
    return ConditionUtil.evaluateString(accountLocation, operator, value);
   }

  /**
   * Evaluate the "accountCountry" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountCountryCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    boolean result = false;
    OrgEntityManageBean orgEntityManageBean = getOrgEntityManageBean();
    Vector attributeVector = orgEntityManageBean.getAttribute(MEMBER_ATTRIBUTE_COUNTRIES_OF_OPERATION, null);
    if (attributeVector != null)
     {
      for (int i = 0; i < attributeVector.size(); i++)
       {
        if (attributeVector.elementAt(i).equals(value))
         {
          result = true;
          break;
         }
       }
     }
    if (operator.equals(OPERATOR_NOT_EQUAL_TO))
     {
      result = !result;
     }
    return result;
   }

  /**
   * Evaluate the "accountBusiness" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountBusinessCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String accountBusiness = "";
    OrgEntityManageBean orgEntityManageBean = getOrgEntityManageBean();
    Vector attributeVector = orgEntityManageBean.getAttribute(MEMBER_ATTRIBUTE_ORG_ENTITY_BUSINESS_TYPE, null);
    if (attributeVector != null && attributeVector.size() > 0)
     {
      accountBusiness = (String) attributeVector.elementAt(0);
     }
    return ConditionUtil.evaluateString(accountBusiness, operator, value);
   }

  /**
   * Evaluate the "accountIndustry" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountIndustryCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    String accountIndustry = "";
    OrgEntityManageBean orgEntityManageBean = getOrgEntityManageBean();
    Vector attributeVector = orgEntityManageBean.getAttribute(MEMBER_ATTRIBUTE_ORG_ENTITY_INDUSTRY_TYPE, null);
    if (attributeVector != null && attributeVector.size() > 0)
     {
      accountIndustry = (String) attributeVector.elementAt(0);
     }
    return ConditionUtil.evaluateString(accountIndustry, operator, value);
   }

  /**
   * Evaluate the "accountCredit" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountCreditCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    boolean result = false;
    return result;
   }

  /**
   * Evaluate the "accountAmountSpent" condition.
   *
   * @param operator  The name of the operator.
   * @param value  The data for the value.
   * @param qualifiers  An array of qualifiers.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateAccountAmountSpentCondition(String operator, String value, SimpleCondition.Qualifier[] qualifiers)
   {
    BigDecimal amountSpent = new BigDecimal(0.0d);
    try
     {
      String parentMemberId = _user.getParentMemberId();
      if (parentMemberId != null)
       {
        StoreAccessBean storeAccessBean = WcsApp.storeRegistry.find(getStoreId());
        CurrencyManager currencyManager = CurrencyManager.getInstance();
        String defaultCurrency = currencyManager.getDefaultCurrency(storeAccessBean);
        Enumeration list = new OrderAccessBean().findWithPushDownQuery("STATUS NOT IN ('P','X','I','E','W','N','M') AND ORGENTITY_ID=" + parentMemberId + " AND STOREENT_ID=" + getStoreId());
        while (list.hasMoreElements())
         {
          OrderAccessBean orderAccessBean = (OrderAccessBean) list.nextElement();
          String orderCurrency = orderAccessBean.getCurrency();
          BigDecimal orderAmount = orderAccessBean.getTotalProductPriceInEJBType();
          BigDecimal adjustment = orderAccessBean.getTotalAdjustmentInEJBType();
          if (adjustment != null)
           {
            orderAmount = orderAmount.add(adjustment);
           }
          if (orderCurrency.equals(defaultCurrency))
           {
            amountSpent = amountSpent.add(orderAmount);
           }
          else
           {
            MonetaryAmount convertedAmount = currencyManager.convert(new MonetaryAmount(orderAmount, orderCurrency), storeAccessBean, defaultCurrency);
            amountSpent = amountSpent.add(convertedAmount.getValue());
           }
         }
       }
     }
    catch (Exception e)
     {
     }
    return ConditionUtil.evaluateBigDecimal(amountSpent, operator, SegmentUtil.toBigDecimal(value));
   }

  /**
   * Evaluate the "roleList" condition.
   *
   * @param parameters  An array of parameters.
   *
   * @return  <code>true</code> if the condition evaluates to true.
   */
  protected boolean evaluateRoleListCondition(OpenCondition.Parameter[] parameters)
   {
    boolean result = false;
    String org = null;
    String orgDN = null;
    Vector targetRoles = new Vector();
    if (parameters != null)
     {
      for (int i = 0; i < parameters.length; i++)
       {
        String parameterName = parameters[i].getName();
        if (OPEN_CONDITION_PARAMETER_ORG.equals(parameterName))
         {
          org = parameters[i].getValue();
         }
        else if (OPEN_CONDITION_PARAMETER_ORG_DN.equals(parameterName))
         {
          orgDN = parameters[i].getValue();
         }
        else if (OPEN_CONDITION_PARAMETER_ROLE.equals(parameterName))
         {
          targetRoles.addElement(parameters[i].getValue());
         }
       }
     }
    Vector roles = null;
    if (org != null)
     {
      Long orgId = null;
      if (VALUE_STORE.equals(org))
       {
        try
         {
          orgId = getCommandContext().getStore().getOwner();
         }
        catch (Exception e)
         {
         }
       }
      else
       {
        try
         {
          orgId = new Long(org);
         }
        catch (Exception e)
         {
         }
       }
      roles = getRoles(orgId);
     }
    else if (orgDN != null)
     {
      try
       {
        OrganizationAccessBean organizationAccessBean = new OrganizationAccessBean().findByDN(orgDN);
        roles = getRoles(organizationAccessBean.getOrganizationIdInEJBType());
       }
      catch (Exception e)
       {
       }
     }
    else
     {
      roles = getRoles();
     }
    if (roles != null)
     {
      for (int i = 0; i < roles.size(); i++)
       {
        for (int j = 0; j < targetRoles.size(); j++)
         {
          if (roles.elementAt(i).equals(targetRoles.elementAt(j)))
           {
            result = true;
           }
         }
       }
     }
    return result;
   }

  /**
   * Get the demographics access bean for the user.
   *
   * @return  The demographics access bean for the user.
   */
  protected DemographicsAccessBean getDemographics()
   {
    if (_demographics == null)
     {
      try
       {
        _demographics = _user.getDemographics();
       }
      catch (Exception e)
       {
       }
     }
    return _demographics;
   }

  /**
   * Get the address access bean for the user.
   *
   * @return  The address access bean for the user.
   */
  protected AddressAccessBean getAddress()
   {
    if (_address == null)
     {
      try
       {
        _address = new AddressAccessBean().findSelfAddressByMember(_user.getMemberIdInEJBType());
       }
      catch (Exception e)
       {
       }
     }
    return _address;
   }

  /**
   * Get the user profile access bean for the user.
   *
   * @return  The user profile access bean for the user.
   */
  protected UserProfileAccessBean getUserProfile()
   {
    if (_userProfile == null)
     {
      try
       {
        _userProfile = _user.getUserProfile();
       }
      catch (Exception e)
       {
       }
     }
    return _userProfile;
   }

  /**
   * Get all the member groups for the user.
   *
   * @return  The member groups for the user.
   */
  protected Vector getMemberGroups()
   {
    if (_memberGroups == null)
     {
      _memberGroups = new Vector();
      try
       {
        Enumeration list = new MemberGroupMemberAccessBean().findAllStoreGroupMember(getStoreId(),
                                                                                     _user.getMemberIdInEJBType());
        while (list.hasMoreElements())
         {
          MemberGroupMemberAccessBean memberGroupMemberAccessBean = (MemberGroupMemberAccessBean) list.nextElement();
          MemberGroupAccessBean memberGroupAccessBean = new MemberGroupAccessBean();
          memberGroupAccessBean.setInitKey_MemberId(memberGroupMemberAccessBean.getMbrGrpId());
          _memberGroups.addElement(memberGroupAccessBean.getMbrGrpName());
         }
       }
      catch (Exception e)
       {
       }
     }
    return _memberGroups;
   }

  /**
   * Get the roles for the user.
   *
   * @return  The roles for the user.
   */
  protected Vector getRoles()
   {
    if (_roles == null)
     {
      _roles = new Vector();
      try
       {
        Enumeration list = new MemberRoleAccessBean().findByMemberId(_user.getMemberIdInEJBType());
        while (list.hasMoreElements())
         {
          MemberRoleAccessBean memberRoleAccessBean = (MemberRoleAccessBean) list.nextElement();
          RoleAccessBean roleAccessBean = new RoleAccessBean();
          roleAccessBean.setInitKey_RoleId(memberRoleAccessBean.getRoleId());
          _roles.addElement(roleAccessBean.getName());
         }
       }
      catch (Exception e)
       {
       }
     }
    return _roles;
   }

  /**
   * Get the roles for the user and a specified organization.
   *
   * @param org  The organization id.
   *
   * @returns The roles for the user.
   */
  protected Vector getRoles(Long orgId)
   {
    if (orgId == null)
     {
      return null;
     }
    if (!orgId.equals(_orgRolesOrgId))
     {
      _orgRolesOrgId = orgId;
      _orgRoles = new Vector();
      try
       {
        Enumeration list = new MemberRoleAccessBean().findByMemberIdOrgEntityId(_user.getMemberIdInEJBType(), orgId);
        while (list.hasMoreElements())
         {
          MemberRoleAccessBean memberRoleAccessBean = (MemberRoleAccessBean) list.nextElement();
          RoleAccessBean roleAccessBean = new RoleAccessBean();
          roleAccessBean.setInitKey_RoleId(memberRoleAccessBean.getRoleId());
          _orgRoles.addElement(roleAccessBean.getName());
         }
       }
      catch (Exception e)
       {
       }
     }
    return _orgRoles;
   }

  /**
   * Get an instance of the CheckUserInMemberGroupCmd task command.
   *
   * @return  An instance of the CheckUserInMemberGroupCmd task command.
   */
  protected CheckUserInMemberGroupCmd getCheckUserInMemberGroupCmd()
   {
    if (_checkUserInMemberGroupCmd == null)
     {
      try
       {
        _checkUserInMemberGroupCmd = (CheckUserInMemberGroupCmd)
          CommandFactory.createCommand(CheckUserInMemberGroupCmd.Name, getStoreId());
        _checkUserInMemberGroupCmd.setCommandContext(getCommandContext());
        if (_checkUserInMemberGroupCmd instanceof CheckUserInMemberGroupCmdImpl)
         {
          CheckUserInMemberGroupCmdImpl checkUserInMemberGroupCmdImpl = (CheckUserInMemberGroupCmdImpl) _checkUserInMemberGroupCmd;
          checkUserInMemberGroupCmdImpl._parent = this;
         }
       }
      catch (Exception e)
       {
       }
     }
    return _checkUserInMemberGroupCmd;
   }

  /**
   * Gets the organization ID for the specified account.
   *
   * @param accountName
   *
   * @return An organization ID.
   */
  protected Long getAccountOrganizationId(String accountName)
   {
    try
     {
      AccountAccessBean accountAccessBean = new AccountAccessBean().findByNameAndMemberId(accountName, getCommandContext().getStore().getOwner());
      Enumeration list = new ParticipantAccessBean().findByTradingAndRole(accountAccessBean.getAccountIdInEJBType(), ECContractConstants.EC_PARTICIPANT_ROLE_ACCOUNTHOLDER);
      if (list.hasMoreElements())
       {
        ParticipantAccessBean participantAccessBean = (ParticipantAccessBean) list.nextElement();
        return participantAccessBean.getMemberIdInEJBType();
       }
     }
    catch (Exception e)
     {
     }
    return null;
   }

  /**
   * Get an instance of UserManageBean for the user.
   *
   * @return An instance of UserManageBean.
   */
  protected UserManageBean getUserManageBean()
   {
    if (_userManageBean == null)
     {
      _userManageBean = new UserManageBean();
      try
       {
        _userManageBean.setInitKey(_user.getMemberId());
        _userManageBean.refreshCopyHelper();
       }
      catch (Exception e)
       {
       }
     }
    return _userManageBean;
   }

  /**
   * Get an instance of OrgEntityManageBean for the user's organization.
   *
   * @return An instance of OrgEntityManageBean.
   */
  protected OrgEntityManageBean getOrgEntityManageBean()
   {
    if (_orgEntityManageBean == null)
     {
      _orgEntityManageBean = new OrgEntityManageBean();
      try
       {
        _orgEntityManageBean.setInitKey(_user.getParentMemberId());
        _orgEntityManageBean.refreshCopyHelper();
       }
      catch (Exception e)
       {
       }
     }
    return _orgEntityManageBean;
   }
 }
