//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.order.commands.*;

import com.ibm.commerce.command.*;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.utils.*;
import com.ibm.util.*;
import java.math.*;
import java.rmi.*;
import java.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.ubf.event.*;
import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.contract.helper.*;
import com.ibm.commerce.price.utils.*;
import com.ibm.commerce.payment.commands.*;

/**
 * Check if the order requires approval.
 */
public class OrderProcessCheckApprovalCmdImpl extends TaskCommandImpl implements OrderProcessCheckApprovalCmd
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	
	private OrderFlowEntityBean orderEntityBean = null;
	private String eventIdentifier = null;
	private BusinessFlowEventData eventData = null;
/**
 * This method implemented the business logic of whether the order requires approval.
 * For each order item in the order, check the contract in the order item. If there is an OrderApprovalTC,
 * it check the amount specified the TC. If the amount is less than the total price of the order 
 * inlcuding shipping and tax, it returns true. If all order items are checked and cannot find any
 * OrderApprovalTCs satisfy the above condition, it returns false.
 */
public boolean checkApproval() throws ECException {
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval");

	try {
		BigDecimal productPrice = orderEntityBean.getTotalProductPriceInEJBType();
		BigDecimal ajustmentPrice = orderEntityBean.getTotalAdjustmentInEJBType();
		BigDecimal shippingCharge = orderEntityBean.getTotalShippingChargeInEJBType();
		BigDecimal shippingTaxCharge = orderEntityBean.getTotalShippingTaxInEJBType();
		BigDecimal tax = orderEntityBean.getTotalTaxInEJBType();
		double valueInOrder = 0;
		if (productPrice != null)
			valueInOrder += productPrice.doubleValue();
		if (ajustmentPrice != null)
			valueInOrder += ajustmentPrice.doubleValue();
		if (shippingCharge != null)
			valueInOrder += shippingCharge.doubleValue();
		if (shippingTaxCharge != null)
			valueInOrder += shippingTaxCharge.doubleValue();
		if (tax != null)
			valueInOrder += tax.doubleValue();
			
		String currencyInOrder = orderEntityBean.getCurrency();
		
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval", "Product total amount " + valueInOrder + " with currency " +  currencyInOrder);

		CurrencyManager cm = CurrencyManager.getInstance();
		OrderItemAccessBean [] orderItemAB = orderEntityBean.getOrderItems();

	  if (orderItemAB != null) {
		for (int i = 0; i < orderItemAB.length; i++) {
			String tradingId = orderItemAB[i].getContractId();

			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval", "Trading id in the order item " + tradingId);

			TradingAgreementAccessBean tradingAgreementAB = new TradingAgreementAccessBean();
			tradingAgreementAB.setInitKey_tradingId(tradingId);
			TermConditionAccessBean[] tc = tradingAgreementAB.getTCsByTCSubType(com.ibm.commerce.contract.helper.ECContractConstants.EC_ELE_ORDERAPPROVAL_TC, getUserId());
		  if (tc != null) {
			for (int j = 0; j < tc.length; j++) {
				String id = tc[j].getReferenceNumber();

				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval", "Found approval TC with id " + id);

				OrderApprovalTCAccessBean approvalTCAB = new OrderApprovalTCAccessBean();
				approvalTCAB.setInitKey_referenceNumber(id);
				MonetaryAmount amountInTC = new MonetaryAmount(approvalTCAB.getValueInEJBType(), approvalTCAB.getCurrency());
				MonetaryAmount convertedAmount = amountInTC;

				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval", "The amount in TC " + amountInTC.getValue() + " with currency " +  amountInTC.getCurrency());

				if (!currencyInOrder.equals(amountInTC.getCurrency())) {
					convertedAmount = cm.convert(amountInTC, commandContext.getStore(), currencyInOrder);
				}
				if (valueInOrder >= convertedAmount.getValue().doubleValue()) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval", "Approval required");
					return true;
				}
			}
		  }
		}
	  }
	} catch (CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), "checkApproval", ex);
	} catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), "checkApproval", ex);
	} catch (NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, this.getClass().getName(), "checkApproval", ex);
	} catch (RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), "checkApproval", ex);
	}
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkApproval");

	return false;
}
/**
 * Allow the entry action to return an eventIdentifier to start an automatic transition.
 * @return The eventIdentifier for the automatic transition.
 */
public String getEventIdentifier()
{
	return eventIdentifier;
}
/*
 * This is a helper method that merges new nvps (hshFrom) with old nvps (hshTo)
 *
 * if value of a new nvp is null or empty string remove nvp with this name from
 * an old set (hshTo).
 */
private void mergeNVPs(TypedProperty hshFrom, TypedProperty hshTo) throws ParameterNotFoundException, InvalidParameterValueException
{
	Enumeration enNames = hshFrom.keys();
	while (enNames.hasMoreElements())
	{
		String    strName      = (String   )enNames.nextElement();
		String [] arrstrValues = hshFrom.getArray(strName, null);
		if (arrstrValues == null || arrstrValues[0] == null || arrstrValues[0].length() == 0)
		  hshTo.remove(strName);
		else
		  hshTo.put(strName, arrstrValues);
	}
	
}
/**
 * Calls checkApproval() to find out if the approval is required.
 * if checkApproval() returns true, it sets the eventIdentifier to be "BeginApproval".
 * if checkApproval() returns false, it sets the eventIdentifier to be "NoApprovalNeeded".
 */
public void performExecute() throws ECException {
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "performExecute");
//	checkParameters();

	if (checkApproval() == true) {
		// check payment parameters now, otherwise the error will be detected when the approver approves the order.
		TypedProperty req = new TypedProperty();
		try {
			req = retrieveNVPs(orderEntityBean, getCommandContext().getRequestProperties());
		} catch (Exception ex) {}
		PaymentHelper.getInstance().checkPaymentInfoForDelayedProcess(
			req, orderEntityBean, null, getCommandContext());

		eventIdentifier = OrderConstants.EC_ORDER_FLOW_TRANSIT_EVENT_BEGIN_APPROVAL;
	} else {
		eventIdentifier = OrderConstants.EC_ORDER_FLOW_TRANSIT_EVENT_AUTO_APPROVE;
	}
//	eventIdentifier = OrderConstants.EC_ORDER_FLOW_TRANSIT_EVENT_AUTO_REJECT;

	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "performExecute", "eventIdentifier returned: " + eventIdentifier);
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "performExecute");
}
public void reset()
{
	orderEntityBean = null;
	eventIdentifier = null;
	eventData = null;
}
/**
 * This is a helper method that retrieves the NVPs from the Order Pay Info table.
 */
private TypedProperty retrieveNVPs(OrderAccessBean orderAB, TypedProperty newReq) throws ECException, NamingException, FinderException, RemoteException, CreateException
{
	TypedProperty oldReq = MiscCmd.getOrderPaymentInfo(orderAB, commandContext);

	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "retrieveNVPs", "The NVPs retrieved from ordpayinfo:\n" + oldReq.toProtectedString());

	mergeNVPs( newReq, oldReq);
	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "retrieveNVPs", "The NVPs merged with the new request properties:\n" + oldReq.toProtectedString());
		
	return oldReq;
}
/**
 * The BusinessFlowEventListener will use this method to pass in all data the command needs.  .
 * @param anEventData The business flow event data which includes all information, including requestPropoerties,
 * commandContext etc.
 */
public void setBusinessFlowEventData(com.ibm.commerce.ubf.event.BusinessFlowEventData anEventData)
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setBusinessFlowEventData");
	eventData = anEventData;
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setBusinessFlowEventData");
}
/**
 * Allow UBF passes an entity Id to the entry action command.  
 * @param anEntityId The entity Id.
 */
public void setEntityId(Long anEntityId)
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setEntityId");
	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setEntityId", "Set entity id " + anEntityId);
	if (orderEntityBean == null) {
		orderEntityBean = new OrderFlowEntityBean();
		orderEntityBean.setInitKey_orderId(anEntityId.toString());
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setEntityId");
}
/**
 * Allow UBF passes an entity object to the entry action command to avoid instantiating another instance of entity object.   
 * @param anEntityObject The entity object.
 */
public void setEntityObject(Object anEntityObject)
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setEntityObject");
	orderEntityBean = (OrderFlowEntityBean) anEntityObject;
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "setEntityObject");
}
public void validateParameters() throws ECException {
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkParameters");
	if (orderEntityBean == null) {
		throw new ECApplicationException(ECMessage._ERR_DIDNT_SET_ORDER, this.getClass().getName(), "checkParameters");
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "checkParameters");
}
}
