//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.orderitems.commands.*;

import com.ibm.commerce.datatype.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.inventory.objects.*;
import com.ibm.commerce.common.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.price.utils.*;
import com.ibm.commerce.price.commands.*;
import java.math.*;


/**
 * Creates the componets that are associated with a composite order item.  
 * It derives the component list in one of two ways
 * <UL>
 * <LI>Uses a list of CatalogEntries that are passed in.</LI>
 * <LI>Derives the list of components by finding all of the unassigned componets through the given configuration id.</LI>
 * </UL>
 * Only one of the ConfigurationId and the Components are required.
 * Quantity is only required if a component list is provided 
 * If the componets are explicitly set, then they are used, otherwise, they are dervied from the ConfigurationId.
 * <p> 
 */
public class AddOrderItemComponentsCmdImpl  extends TaskCommandImpl implements AddOrderItemComponentsCmd
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	String				istrConfigurationId		= null;
	Long[]	 			iarnCatentryComponets 	= null;
	Double[]			iarnQuantity			= null;
	String				istrCurrency			= null;
	BigDecimal[]		iarnPrice				= null;
	OrderItemAccessBean iabOrderItem 			= null;
	Long				inOrderItemId 			= null;
	Vector				ivecOutComponents		= null;

private void addOutputComponent(OrderItemComponentAccessBean abOIComp) 
{
	//add the component to the output component list
	ivecOutComponents.add(abOIComp);
}
private Integer convertQuantity(Long anCatalogentryId, Double anCatalogInventory, Long anItemSpecId) throws ECException
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "convertQuantity");
	
	//coverts a catalog quantity into a itemspec quantity
	final String methodName = "convertQuantity";
	com.ibm.commerce.price.utils.QuantityAmount quantity = null;

	try {

		CatalogEntryShippingAccessBean catEntShipAB = Helper.getCatalogEntryShippingAB(anCatalogentryId);
		quantity = Helper.createRoundedQuantityAmount(catEntShipAB, anCatalogInventory.doubleValue());

		// get base item id
		ItemSpecificationAccessBean itemSpcAB = new ItemSpecificationAccessBean();
		itemSpcAB.setInitKey_itemspcId(anItemSpecId.toString());

		String baseItemId = itemSpcAB.getBaseItemId();

		BaseItemAccessBean baseItemAB = new BaseItemAccessBean();
		baseItemAB.setInitKey_baseItemId(baseItemId);

		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "convertQuantity");
		
		return Helper.calculateInventoryQuantity(quantity, baseItemAB);
		
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, ex);
	}
}
private void createComponents(String astrConfigurationId, Long[] anComponets, Double[] anQuantity, String astrCurrency, BigDecimal[] aarnPrice, OrderItemAccessBean aabOrderItem) throws ECException
{

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "createComponents");

	//check to make sure that each component has an associated quantity
	if ((anComponets.length != anQuantity.length) || (aarnPrice != null && aarnPrice.length != anComponets.length) )
	{
		Object[] msgParms = {astrConfigurationId};
		throw new ECApplicationException(ECMessage._ERR_MISSING_COMPONENT_INFO, this.getClass().getName(), "createComponents", msgParms);
	}

	//check to make sure that if the prices have been set then there is a currency
	if (aarnPrice != null && astrCurrency == null)
	{
		Object[] msgParms = {astrConfigurationId};
		throw new ECApplicationException(ECMessage._ERR_NO_CURRENCY_FOR_COMPONENTS, this.getClass().getName(), "createComponents", msgParms);
	}

		CatalogEntryAccessBean abCatentry = null;
		OrderItemComponentAccessBean abOIComponent = null;
		Long[]		nItemSpec 		= new Long[anComponets.length];
		Integer[]	nItemSpecQuantities = new Integer[anComponets.length];


		try
		{
			//first, if a components already exists for this configuration id, they need to be be removed
			//the finder method returns a list of components that are associated to the orderitem id specified, 
			//or if the orderitem id is null it returns the components that are associated to no orderitem
			abOIComponent = new OrderItemComponentAccessBean();
			Long nOrderItemId = null;
			if (aabOrderItem != null)
				nOrderItemId = aabOrderItem.getOrderItemIdInEJBType();
			Enumeration e = abOIComponent.findByOrderItemIdAndConfigurationId(nOrderItemId, astrConfigurationId);
			
			while (e.hasMoreElements())
			{
				abOIComponent = (OrderItemComponentAccessBean)e.nextElement();
	
				//This is a old component, we should remove it so that it can be replaced 
				//with the new version
				abOIComponent.getEJBRef().remove();
				
			}
			

			//Create the components
			//resolve catentries to itemspc and catalog quantities to inventory quantities
			for (int x = 0; x < anComponets.length; x++)
			{
				abCatentry = new CatalogEntryAccessBean();
				abCatentry.setInitKey_catalogEntryReferenceNumber(anComponets[x].toString());
				nItemSpec[x] = abCatentry.getItemspc_idInEJBType();
				abOIComponent = new OrderItemComponentAccessBean(astrConfigurationId, nOrderItemId ,nItemSpec[x]);
				abOIComponent.setCatalogEntryId(anComponets[x].toString());
				abOIComponent.setCurrency(astrCurrency);

				if (aarnPrice != null)
					abOIComponent.setUnitPrice(aarnPrice[x].toString());
	
				//We need to normalize the catalog quantity
				abOIComponent.setCatalogQuantity(new Double(Helper.createNormalizedQuantity( Helper.getCatalogEntryShippingAB(anComponets[x]), anQuantity[x].doubleValue())).toString() );
			
				//Check if we are using ATP
				//Note: need to change "0" to the correct constant
				if (!getCommandContext().getStore().getAllocationGoodFor().equals("0"))
				{
					nItemSpecQuantities[x] = convertQuantity(anComponets[x], anQuantity[x], nItemSpec[x]);
					abOIComponent.setInventoryQuantity(nItemSpecQuantities[x].toString());
				}

				abOIComponent.commitCopyHelper();

				//add the new component to the output component list
				addOutputComponent(abOIComponent);
			}
			
	} catch (Exception e) {
		Object[] msgParams = {istrConfigurationId};
		throw new ECSystemException(ECMessage._ERR_COULD_NOT_CREATE_UPDATE_CONFIG_COMPONETS, this.getClass().getName(), "createComponents", msgParams, e);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "createComponents");
}		
/**
 * Gets the output list  of components that were created or updated.
 *
 */
public java.util.Vector getOutComponentList() 
{
	return ivecOutComponents;
}
public void performExecute() throws ECException
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "performExecute");

	OrderItemComponentAccessBean abOICompList = new OrderItemComponentAccessBean();

	try
	{
		super.performExecute();
		ivecOutComponents = new Vector();

		if (istrConfigurationId == null)
		{
			Object[] msgParms = {com.ibm.commerce.order.utils.OrderConstants.EC_CONFIGURATION_ID}; 
			throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, this.getClass().getName(), "performExecute", msgParms);
		}
	
		if (inOrderItemId != null)
		{
			iabOrderItem = new OrderItemAccessBean();
			iabOrderItem.setInitKey_orderItemId(inOrderItemId.toString());
			iabOrderItem.getEJBRef();
		
			if (iarnCatentryComponets != null)
			{
				///create the components and assign them to this orderitem
				createComponents(istrConfigurationId, iarnCatentryComponets, iarnQuantity, istrCurrency, iarnPrice, iabOrderItem);

			}
			else
			{
				//what this really means is that we should "move" the configuration components 
				//to be components of this particular order item
				updateComponents(istrConfigurationId, iabOrderItem);
			}
			
		
		}
		else if (iarnCatentryComponets != null)
		{
			//no orderitem set, so get components from component list and do not assign them to an order item
			//check to make sure that each component has an associated quantity
			//resolve catentries to itemspc and catalog quantities to inventory quantities
			createComponents(istrConfigurationId, iarnCatentryComponets, iarnQuantity, istrCurrency, iarnPrice, null);
		}
		else
		{
			//no component list or order item so throw exception
			Object[] msgParms = {istrConfigurationId};
			throw new ECApplicationException(ECMessage._ERR_COULD_NOT_CREATE_UPDATE_CONFIG_COMPONETS, this.getClass().getName(), "performExecute", msgParms);
		}

	} catch (ECException e) {
		throw e;
	} catch (Exception e) {
		Object[] msgParms = {istrConfigurationId};
		throw new ECSystemException(ECMessage._ERR_COULD_NOT_CREATE_UPDATE_CONFIG_COMPONETS, this.getClass().getName(), "performExecute", msgParms, e);
	}
		

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "performExecute");
}
/**
 * Set a list of Catalog Entries (components) that will be added to the OrderItem.
 *
 */
public void setComponents(Long[] anCatentry) 
{
	iarnCatentryComponets = anCatentry;
}
/**
 * Sets the Configuration id that is used to retrieve the components
 *
 */
public void setConfigurationId(String astrConfigurationId) 
{
	istrConfigurationId = astrConfigurationId;
}
/**
 * (Optional)Sets the currency for the prices of the components.
 */
public void setCurrency(java.lang.String astrCurrency) 
{
	istrCurrency = astrCurrency;
}
/**
 * (Optional) Set an OrderItem that the componets will be added to.
 *
 */
public void setOrderItemId(Long anOrderItemId) 
{
	inOrderItemId = anOrderItemId;
}
/**
 * (Optional) Sets a list of prices for the components that will be added to the OrderItem.
 */
public void setPrices(java.math.BigDecimal[] anPrice) 
{
	iarnPrice = anPrice;
}
/**
 * Set a list of quantities of each Catalog Entry (component) that will be added to the OrderItem.
 *
 */
public void setQuantity(Double[] anQuantity) 
{
	iarnQuantity = anQuantity;
}
private void updateComponents(String astrConfigurationId, OrderItemAccessBean aabOrderItem) throws ECException
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "updateComponents");
	
	//we "move" the configuration components 
	//to be components of this particular order item
	try
	{
			//first we need to check if there are any components waiting to be associated to an orderitem
			//This should only return those components that are associated to no orderitems
			OrderItemComponentAccessBean abOIComponent = new OrderItemComponentAccessBean();
			Enumeration e = abOIComponent.findByOrderItemIdAndConfigurationId(null, astrConfigurationId);
		
			//now, if a components already exists for this configuration id + orderitemid, they need to be be removed
			//if there are components waiting to be associated
			if (e.hasMoreElements())
			{
				abOIComponent = new OrderItemComponentAccessBean();
				Long nOrderItemId = null;
				
				//the finder method returns a list of components that are associated to the orderitem id specified,
				Enumeration enumOIComp = abOIComponent.findByOrderItemIdAndConfigurationId(aabOrderItem.getOrderItemIdInEJBType(), astrConfigurationId);
			
				while (enumOIComp.hasMoreElements())
				{
					abOIComponent = (OrderItemComponentAccessBean)enumOIComp.nextElement();
	
					//This is a old component, we should remove it so that it can be replaced 
					//with the new version
					abOIComponent.getEJBRef().remove();
				}
			}
		
				
		//now we can associate the existing components to this order item
		while (e.hasMoreElements())
		{
			abOIComponent = (OrderItemComponentAccessBean)e.nextElement();
			abOIComponent.setOrderItemId(aabOrderItem.getOrderItemId());
			abOIComponent.commitCopyHelper();

			//add the new component to the output component list
			addOutputComponent(abOIComponent);			
		}

	} catch (Exception e) {
		Object[] msgParms = {astrConfigurationId};
		throw new ECSystemException(ECMessage._ERR_COULD_NOT_CREATE_UPDATE_CONFIG_COMPONETS, this.getClass().getName(), "performExecute", msgParms, e);
	}
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), "updateComponents");

}
}
