//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.usermanagement.commands.*;

import java.util.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;
/**
 * The implementation for the AuditAddrBookCmd task command checks the parameters that are passed 
 * into the AddressAdd and AddressUpdate commands.
 */
public class AuditAddressCmdImpl  extends AuditBaseCmdImpl implements AuditAddressCmd
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	public final static String CLASSNAME = "com.ibm.commerce.usermanagement.commands.AuditAddressCmdImpl";	

	//The last name of the individual. 
	protected String istrLastName = null;
	//The first name of the individual. 
	protected String istrFirstName = null;
	//The first address line of the individual's street address. 
	protected String istrAddress1 = null;
	//The name of the city where the individual resides. 
	protected String istrCity = null;
	//The name of the state, province, or equivalent where the individual resides. 
	protected String istrState = null;
	//The zip code or equivalent postal code of the individual's address. 	
	protected String istrZipCode = null;
	//The name of the country where the individual resides. 
	protected String istrCountry = null;
	
public AuditAddressCmdImpl()
{
	
}
/**
 * Returns the first line of the address.
 *
 * @return The first line of the address
 */
public String getAddress1() {
	return istrAddress1;
}
/**
 * Returns the city name.
 *
 * @return The city name 
 */
public String getCity() {
	return istrCity;
}
/**
 * Returns the country name.
 *
 * @return The country name
 */
public String getCountry() {
	return istrCountry;
}
/**
 * Returns the last name of the individual.
 *
 * @return The last name
 */
public String getLastName() {
	return istrLastName;
}
/**
 * Returns the name of the state, province, or equivalent where the individual resides.
 *
 * @return The name of the state
 */
public String getState() {
	return istrState;
}
/**
 * Returns the zip code or equivalent postal code of the individual's adddres.
 *
 * @return The zip code
 */
public String getZipCode() {
	return istrZipCode;
}
public void performExecute() throws ECException
{
	final String METHODNAME = "performExecute";
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_USER, CLASSNAME, METHODNAME);
	super.performExecute();
/*	
	// Read Properties from property file.
	ihshProperties = PropertyResourceBundleReader.readProperties(istrPropertyFileName, commandContext.getLocale());
	// Check if the parameter is missing.
	if (ihshProperties != null)
	{
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_LASTNAME)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())	
		{
			if (istrLastName != null && istrLastName.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_LASTNAME);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME,
									ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_LASTNAME),
									istrErrorViewName, hshNVPs);
			}
		}
	
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_FIRSTNAME)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())	
		{
			if (istrFirstName != null && istrFirstName.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_FIRSTNAME);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME,
									ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_FIRSTNAME),
									istrErrorViewName, hshNVPs);
			}
		}
	
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_ADDRESS1)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())		
		{
			if (istrAddress1 != null && istrAddress1.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_ADDRESS1);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME,
									 ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_ADDRESS1),
									 istrErrorViewName, hshNVPs);
			}
		}
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_CITY)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())		
		{
			if (istrCity != null && istrCity.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();		
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_CITY);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME,
									 ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_CITY),
									 istrErrorViewName, hshNVPs);
			}
		}
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_STATE)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())		
		{
			if (istrState != null && istrState.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();		
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_STATE);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME, 
									ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_STATE),
									istrErrorViewName, hshNVPs);
			}
		}
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_ZIPCODE)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())		
		{
			if (istrZipCode != null && istrZipCode.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();		
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_ZIPCODE);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME, 
									ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_ZIPCODE),
									istrErrorViewName, hshNVPs);
			}		
		}
		if (((Boolean)((Hashtable)ihshProperties.get(ECUserConstants.EC_ADDR_COUNTRY)).get(ECUserConstants.EC_RB_REQUIRED)).booleanValue())		
		{
			if (istrCountry != null && istrCountry.length() == 0)
			{
				ibResult = false;
				TypedProperty hshNVPs = new TypedProperty();		
				hshNVPs.put(ECConstants.EC_ERROR_CODE, ECUserConstants.EC_ADDR_ERR_BAD_COUNTRY);
				throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, CLASSNAME, METHODNAME,
									 ECMessageHelper.generateMsgParms(ECUserConstants.EC_ADDR_COUNTRY),
									 istrErrorViewName, hshNVPs);
		
			}
		}
	}
*/	
	ibResult = true;
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_USER, CLASSNAME, METHODNAME);
}
/**
 * Sets the first line of the address.
 *
 * @param astrAddress1 The first line of the address
 */
public void setAddress1(String astrAddress1) {
	istrAddress1 = astrAddress1;	
}
/**
 * Sets the city name.
 *
 * @param astrCity The city name
 */
public void setCity(String astrCity)	{
	istrCity = astrCity;
}
/**
 * Sets the country name.
 *
 * @param astrCountry The country name
 */
public void setCountry(String astrCountry) {
	istrCountry = astrCountry;
}
/**
 * Sets the First name of the individual.
 *
 * @param astrFirstName The First name
 */
public void setFirstName(String astrFirstName) {
	istrFirstName = astrFirstName;
}
/**
 * Sets the last name of the individual.
 *
 * @param astrLastName The last name
 */
public void setLastName(String astrLastName) {
	istrLastName = astrLastName;
}
public void setRequestProperties(TypedProperty aRequestProperties) throws ECApplicationException {
	final String METHODNAME="setRequestProperties";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_USER, CLASSNAME, METHODNAME);
	super.setRequestProperties(aRequestProperties);
/*	
	requestProperties = aRequestProperties;
	
	setLastName(aRequestProperties.getString(ECUserConstants.EC_ADDR_LASTNAME, null));
	setFirstName(aRequestProperties.getString(ECUserConstants.EC_ADDR_FIRSTNAME, null));
	setAddress1(aRequestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS1, null));
	setCity(aRequestProperties.getString(ECUserConstants.EC_ADDR_CITY, null));
	setState(aRequestProperties.getString(ECUserConstants.EC_ADDR_STATE, null));
	setCountry(aRequestProperties.getString(ECUserConstants.EC_ADDR_COUNTRY, null));
	setZipCode(aRequestProperties.getString(ECUserConstants.EC_ADDR_ZIPCODE, null));
*/			
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_USER, CLASSNAME, METHODNAME);
}
/**
 * Sets the name of the state, province, or equivalent where the individual resides.
 *
 * @param astrState The name of the state
 */
public void setState(String astrState) {
	istrState = astrState;
}
/**
 * Sets the zip code or equivalent postal code of the individual's adddres.
 *
 * @param astrZipCode The zip code
 */
public void setZipCode(String astrZipCode) {
	istrZipCode = astrZipCode;
}
}
